// Get handles
var loginButton = document.getElementById( 'loginButton' ),
	logoutButton = document.getElementById( 'logoutButton' ),
	offlineIndicator = document.getElementById( 'offline' ),
	userIndicator = document.getElementById( 'user' ),
	elUserName = document.getElementById( 'username' ),
    subscribeButton = document.getElementById( 'subscribeButton' ),
    subscribeIndicator = document.getElementById( 'subscribed' );

var subs = [];

var onOffline = function( res ) {
    offlineIndicator.className = res ? '' : 'hidden';
};

var onLogin = function( res ) {
    console.log( 'Login Callback Fired' );
};

var onSubscribe = function( identifier, status, expiry ) {
    console.log( 'Subscribe Callback Fired' );

    var res = {};
    res[ Banner.SubscriptionStatus.SUCCESS ] = function() {
        console.log( 'subscription succeeded' );
        subscribeIndicator.className = '';
        subscribeButton.className = 'hidden';
    };

    res[ Banner.SubscriptionStatus.CANCELLED ] = function() {
        console.log( 'subscription attempt cancelled' );
        document.querySelector( '#subscribeButton span' ).className = '';
        document.querySelector( '#subscribingIcon' ).className = 'hidden';
    };

    res[ Banner.SubscriptionStatus.FAILED ] = function() {
        console.log( 'subscription failed' );
        document.querySelector( '#subscribeButton span' ).className = '';
        document.querySelector( '#subscribingIcon' ).className = 'hidden';
        subscribe.attemptSubscription( true );
    };

    res[ '_default' ] = function() {
        console.error( 'subscription status code not recognised' );
    };

    _.contains( _.keys( res ), status.toString() ) ? res[ status ](): res._default();
};

var subscribe = ( function() {
	var ids = [];

    return {
        set: function( subscriptions ) {
            _.each( subscriptions, function( sub ) {
              	ids.push( sub.identifier );
            });
        },

        get: function() {
          	return ids;
        },

        // @param newID {Boolean} if true removes the id that just failed so that next time a new one will be used
        // @todo using newID like this is pretty poor as it leads to an additional failing call.  FIX!
        attemptSubscription: function( newID ) {
            if ( ids.length ) {
                Banner.subscribe( newID ? ids.shift() : ids[ 0 ], onSubscribe );
                document.querySelector( '#subscribeButton span' ).className = 'hidden';
                document.querySelector( '#subscribingIcon' ).className = '';
                return;
            }

            subscribeButton.className = 'disabled';
            console.error( 'No valid subscription identifiers found' );
        }
    }

})();

var onGetSubscriptions = function( res ) {
    var i = 0;
    if ( res.length ) {
        for ( ; i < res.length; i++ ) {
            subs.push( res[ i ] );
        }

        subscribe.set( subs );
    }

    subscribeButton.className = res.length ? '' : 'hidden';
}


// Respond to change in login status
var onLoginChange = function( res ) {
    console.log( 'login change' );
    console.log( res.userName ? 'logged in as ' + res.userName : 'currently logged out' );

    function updateUsername( username ) {
        elUserName.innerHTML = username;
    }

    function showLogin() {
        loginButton.className = '';
        logoutButton.className = 'hidden';
        userIndicator.className = 'hidden';
    }

    function hideLogin() {
        loginButton.className = 'hidden';
        logoutButton.className = '';
        userIndicator.className = '';
        updateUsername( res.userName );
    }

    res.userName ? hideLogin() : showLogin();
}



// Fire into the API
Banner.offline( onOffline );
Banner.loginStateChanged( onLoginChange );
Banner.getSubscriptions( onGetSubscriptions );
Banner.getIssues( 1, function( res ) {
    console.log( 'Fired getIssues callback' );
});
